--[[
Copyright (C) 2021 courseplay dev team
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]


VehicleSwapExtended = {}

local VehicleSwapExtended_mt = Class(VehicleSwapExtended);
addModEventListener(VehicleSwapExtended)

VehicleSwapExtended.MOD_DIRECTORY = g_currentModDirectory
VehicleSwapExtended.MOD_NAME = g_currentModName

VehicleSwapExtended.MODE_ALL_VEHICLES = 1
VehicleSwapExtended.MODE_OWNED_VEHICLES = 2
VehicleSwapExtended.MODE_DISABLED = 3
--static variables
VehicleSwapExtended.currentMode = VehicleSwapExtended.MODE_OWNED_VEHICLES

VehicleSwapExtended.disabledVehicles = {}
VehicleSwapExtended.numDisabledVehicles = 0
function VehicleSwapExtended:loadMap(name)
	if g_client then
		---Action event texts
		VehicleSwapExtended.texts = {
			allVehicles = g_i18n:getText("VEHICLE_SWAP_EXTENDED_SWITCH_MODE_ALL"),
			ownedVehicles = g_i18n:getText("VEHICLE_SWAP_EXTENDED_SWITCH_MODE_OWNED"),
			disabled =  g_i18n:getText("VEHICLE_SWAP_EXTENDED_SWITCH_MODE_DISABLED"),
			resetDisabledVehicles = g_i18n:getText("VEHICLE_SWAP_EXTENDED_RESET_DISABLED_VEHICLES")
		}
		---Load last currentMode
		local mode = Utils.getNoNil(getXMLInt(g_savegameXML, "gameSettings.vehicleSwapExtended#currentMode"), VehicleSwapExtended.MODE_ALL_VEHICLES)
		VehicleSwapExtended.setNextSwapMode(mode)
		VehicleSwapExtended.updateChangeSwapModeActionEventState()
	end
end

function registerActionEvents()
	if g_client then
        local _, eventId = g_inputBinding:registerActionEvent(InputAction.VEHICLE_SWAP_EXTENDED_SWITCH_MODE, VehicleSwapExtended, VehicleSwapExtended.actionEventChangeSwapMode, false, true, false, true)
		g_inputBinding:setActionEventTextPriority(eventId, GS_PRIO_NORMAL)
        VehicleSwapExtended.switchModeEventId = eventId
		VehicleSwapExtended.updateChangeSwapModeActionEventState()

		local _, eventId = g_inputBinding:registerActionEvent(InputAction.VEHICLE_SWAP_EXTENDED_RESET_DISABLED_VEHICLES, VehicleSwapExtended, VehicleSwapExtended.actionEventResetDisabledVehicles, false, true, false, true)
		g_inputBinding:setActionEventTextPriority(eventId, GS_PRIO_NORMAL)
		g_inputBinding:setActionEventText(eventId, VehicleSwapExtended.texts.resetDisabledVehicles)
        VehicleSwapExtended.resetDisabledVehiclesEventId = eventId
		VehicleSwapExtended.updateResetDisabledVehiclesActionEventState()
    end
end
FSBaseMission.registerActionEvents = Utils.appendedFunction(FSBaseMission.registerActionEvents, registerActionEvents)

---Action event for change vehicle swap state
---@param class self 
---@param class actionName 
---@param class inputValue 
---@param class callbackState 
---@param class isAnalog 
function VehicleSwapExtended.actionEventChangeSwapMode(self, actionName, inputValue, callbackState, isAnalog)
	VehicleSwapExtended.setNextSwapMode()
	VehicleSwapExtended.updateChangeSwapModeActionEventState()
end

---Action event for resetting all disabled swapping vehicles
---@param class self 
---@param class actionName 
---@param class inputValue 
---@param class callbackState 
---@param class isAnalog 
function VehicleSwapExtended.actionEventResetDisabledVehicles(self, actionName, inputValue, callbackState, isAnalog)
	for vehicle,_ in pairs(VehicleSwapExtended.disabledVehicles) do 
		vehicle:resetSwappingAllowed()
	end
	VehicleSwapExtended.numDisabledVehicles = 0
	VehicleSwapExtended.disabledVehicles = {}
	VehicleSwapExtended.updateResetDisabledVehiclesActionEventState()
end

function VehicleSwapExtended.addDisabledVehicle(vehicle)
	VehicleSwapExtended.disabledVehicles[vehicle] = true
	VehicleSwapExtended.numDisabledVehicles = VehicleSwapExtended.numDisabledVehicles + 1
	VehicleSwapExtended.updateResetDisabledVehiclesActionEventState()
end

function VehicleSwapExtended.removeDisabledVehicle(vehicle)
	VehicleSwapExtended.disabledVehicles[vehicle] = false
	VehicleSwapExtended.numDisabledVehicles = VehicleSwapExtended.numDisabledVehicles - 1
	VehicleSwapExtended.updateResetDisabledVehiclesActionEventState()
end

---Updates change vehicle swap state action event
---@param class self 
function VehicleSwapExtended.updateChangeSwapModeActionEventState()
    ---Activate/deactivate the VehicleSwapExtended action event 
	if VehicleSwapExtended.getIsSwapModeAllVehicles() then 
		g_inputBinding:setActionEventText(VehicleSwapExtended.switchModeEventId, VehicleSwapExtended.texts.allVehicles)
	elseif VehicleSwapExtended.getIsSwapModeDisabled() then 
		g_inputBinding:setActionEventText(VehicleSwapExtended.switchModeEventId, VehicleSwapExtended.texts.disabled)
	else 
		g_inputBinding:setActionEventText(VehicleSwapExtended.switchModeEventId, VehicleSwapExtended.texts.ownedVehicles)
	end
end

---Updates reset disabled vehicles action event
---@param class self 
function VehicleSwapExtended.updateResetDisabledVehiclesActionEventState()
	local areThereAnyDisabledVehicles = VehicleSwapExtended.numDisabledVehicles>0
	g_inputBinding:setActionEventActive(VehicleSwapExtended.resetDisabledVehiclesEventId, areThereAnyDisabledVehicles)
end

---Sets the next current vehicle swap state
---@param int optional next mode to force this mode, 
---					   default is changing to the next mode
function VehicleSwapExtended.setNextSwapMode(nextMode)
	if not nextMode then 
		nextMode = MathUtil.clamp(VehicleSwapExtended.currentMode+1,VehicleSwapExtended.MODE_ALL_VEHICLES,VehicleSwapExtended.MODE_DISABLED)
		if not g_currentMission.missionDynamicInfo.isMultiplayer then 
			if nextMode == VehicleSwapExtended.MODE_OWNED_VEHICLES then 
				nextMode = nextMode + 1
			end
		end
		if nextMode == VehicleSwapExtended.currentMode then 
			nextMode = VehicleSwapExtended.MODE_ALL_VEHICLES
		end
	else
		if not g_currentMission.missionDynamicInfo.isMultiplayer then 
			if nextMode == VehicleSwapExtended.MODE_OWNED_VEHICLES then 
				nextMode = VehicleSwapExtended.MODE_ALL_VEHICLES
			end
		end
	end
	VehicleSwapExtended.currentMode = nextMode
end

---Is current mode: MODE_ALL_VEHICLES ?
function VehicleSwapExtended.getIsSwapModeAllVehicles()
	return VehicleSwapExtended.currentMode == VehicleSwapExtended.MODE_ALL_VEHICLES
end

---Is current mode: MODE_DISABLED ?
function VehicleSwapExtended.getIsSwapModeDisabled()
	return VehicleSwapExtended.currentMode == VehicleSwapExtended.MODE_DISABLED
end

function VehicleSwapExtended:getIsTabbable(superFunc)
	---Swap is deactivated	
	if VehicleSwapExtended.getIsSwapModeDisabled() then 
		return false
	end
	---No override is needed
	if VehicleSwapExtended.getIsSwapModeAllVehicles() then 
		return superFunc(self)
	end
	---Only allow vehicles that are owned by the current player farmId
	local playerFarmId = g_currentMission.player.farmId
	local vehicleFarmId = self.ownerFarmId
	if playerFarmId and vehicleFarmId then 
		if playerFarmId == vehicleFarmId then 
			return superFunc(self)
		else 
			return false
		end
	end	
	return false
end
Enterable.getIsTabbable = Utils.overwrittenFunction(Enterable.getIsTabbable, VehicleSwapExtended.getIsTabbable)

---Force update action event text after tabbing into a vehicle
function VehicleSwapExtended:enterVehicle(isControlling, playerStyle, farmId)
	VehicleSwapExtended.updateChangeSwapModeActionEventState()
end
Enterable.enterVehicle = Utils.appendedFunction(Enterable.enterVehicle, VehicleSwapExtended.enterVehicle)

function VehicleSwapExtended:deleteMap()
	setXMLInt(g_savegameXML, "gameSettings.vehicleSwapExtended#currentMode", VehicleSwapExtended.currentMode)
	g_gameSettings:saveToXMLFile(g_savegameXML)
end

---Register the spec_vehicleSwapExtendedSpec in all enterable entities
function validateVehicleTypes(vehicleTypeManager)
	for typeName, typeEntry in pairs(vehicleTypeManager:getVehicleTypes()) do
		if SpecializationUtil.hasSpecialization(Enterable, typeEntry.specializations) then
			vehicleTypeManager:addSpecialization(typeName, VehicleSwapExtended.MOD_NAME .. ".vehicleSwapExtendedSpec")	
		end
    end
end
VehicleTypeManager.validateVehicleTypes = Utils.prependedFunction(VehicleTypeManager.validateVehicleTypes, validateVehicleTypes)


