--[[
Copyright (C) 2021 courseplay dev team
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]


VehicleSwapExtendedSpec = {}

VehicleSwapExtendedSpec.MOD_NAME = g_currentModName

function VehicleSwapExtendedSpec.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Enterable, specializations) 
end

function VehicleSwapExtendedSpec.registerEventListeners(vehicleType)	
	SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", VehicleSwapExtendedSpec)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", VehicleSwapExtendedSpec)
end

function VehicleSwapExtendedSpec.registerFunctions(vehicleType)
    SpecializationUtil.registerFunction(vehicleType, "resetSwappingAllowed", VehicleSwapExtendedSpec.resetSwappingAllowed)
end

function VehicleSwapExtendedSpec.registerOverwrittenFunctions(vehicleType)
    SpecializationUtil.registerOverwrittenFunction(vehicleType, "getIsTabbable", VehicleSwapExtendedSpec.getIsTabbable)
end

---Register activate/deactivate swapping into this vehicle action event
---@param bool isActiveForInput
---@param bool isActiveForInputIgnoreSelection
function VehicleSwapExtendedSpec:onRegisterActionEvents(isActiveForInput, isActiveForInputIgnoreSelection)
	if self.isClient then
        local spec = self.spec_vehicleSwapExtendedSpec
        self:clearActionEventsTable(spec.actionEvents)
        if isActiveForInputIgnoreSelection then
            _, actionEventId = self:addActionEvent(spec.actionEvents, InputAction.VEHICLE_SWAP_EXTENDED_SWITCH_VEHICLE, self, VehicleSwapExtendedSpec.actionEvent, false, true, false, true, nil)
            g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_NORMAL)
			spec.actionEventId = actionEventId
            VehicleSwapExtendedSpec.updateActionEventState(self)
        end
    end
end;

---Updates activate/deactivate swapping into this vehicle action event
---@param class self 
function VehicleSwapExtendedSpec.updateActionEventState(self)
	local spec = self.spec_vehicleSwapExtendedSpec
	if spec.isSwappingAllowed then
		g_inputBinding:setActionEventText(spec.actionEventId, spec.texts.tabbingAllowed)
	else 
		g_inputBinding:setActionEventText(spec.actionEventId, spec.texts.tabbingDisallowed)
	end
end

---Action event for activate/deactivate swapping into this vehicle 
---@param class self 
---@param class actionName 
---@param class inputValue 
---@param class callbackState 
---@param class isAnalog 
function VehicleSwapExtendedSpec.actionEvent(self, actionName, inputValue, callbackState, isAnalog)
	local spec = self.spec_vehicleSwapExtendedSpec
	if spec.isSwappingAllowed then 
		spec.isSwappingAllowed = false
		VehicleSwapExtended.addDisabledVehicle(self)
	else 
		spec.isSwappingAllowed = true
		VehicleSwapExtended.removeDisabledVehicle(self)
	end
	VehicleSwapExtendedSpec.updateActionEventState(self)
end


function VehicleSwapExtendedSpec:onLoad(savegame)
    local specName = VehicleSwapExtendedSpec.MOD_NAME .. ".vehicleSwapExtendedSpec"
    self.spec_vehicleSwapExtendedSpec = self["spec_" .. specName]
    local spec = self.spec_vehicleSwapExtendedSpec
    spec.isSwappingAllowed = true
    ---Action event texts
	spec.texts = {}
    spec.texts.tabbingAllowed 		= g_i18n:getText("VEHICLE_SWAP_EXTENDED_SWITCH_VEHICLE_ENABELD")
    spec.texts.tabbingDisallowed	= g_i18n:getText("VEHICLE_SWAP_EXTENDED_SWITCH_VEHICLE_DISABLED")
	if savegame then 
		---Load previous disabled state
		local key = savegame.key..".VehicleSwapExtended.vehicleSwapExtendedSpec"
		spec.isSwappingAllowed = Utils.getNoNil(getXMLBool(savegame.xmlFile, key.."#isSwappingAllowed"),true)
		if not spec.isSwappingAllowed then
			VehicleSwapExtended.addDisabledVehicle(self)
		end
	end
	VehicleSwapExtendedSpec.updateActionEventState(self)
end

function VehicleSwapExtendedSpec:saveToXMLFile(xmlFile, key, usedModNames)
	local spec = self.spec_vehicleSwapExtendedSpec
	setXMLBool(xmlFile, key.."#isSwappingAllowed",Utils.getNoNil(spec.isSwappingAllowed,true))
end

function VehicleSwapExtendedSpec:getIsTabbable(superFunc)
	local spec = self.spec_vehicleSwapExtendedSpec
	if spec.isSwappingAllowed then 
		return superFunc(self)
	end
	return false
end

function VehicleSwapExtendedSpec:resetSwappingAllowed()
	local spec = self.spec_vehicleSwapExtendedSpec
	spec.isSwappingAllowed = true
	VehicleSwapExtendedSpec.updateActionEventState(self)
end